@@ -6,13 +6,13 @@
 #' of cluster membership.
 #'
 #' @details
-#' \code{type} must take one of five values: \code{"hclust"} performs agglomerative
-#' hierarchical clustering via \code{\link{hclust}()}. \code{"kmeans"}
-#' performs k-means clustering via \code{\link{kmeans}()}. \code{"pam"}
-#' performs k-medoids clustering via \code{\link{pam}()}. \code{"dbscan"} performs
-#' density-based clustering via \code{\link{dbscan}()}. \code{"manual"} indicates
-#' that no clustering is necessary and that the researcher has specified cluster
-#' assignments.
+#' \code{type} must take one of six values: \cr
+#' \code{"hclust"}: agglomerative hierarchical clustering with \code{\link{hclust}()}, \cr
+#' \code{"diana"}: divisive hierarchical clustering with \code{\link{diana}()}, \cr
+#' \code{"kmeans"}: k-means clustering with \code{\link{kmeans}()}, \cr
+#' \code{"pam"}: k-medoids clustering with \code{\link{pam}()}, \cr
+#' \code{"dbscan"}: density-based clustering with \code{\link{dbscan}()}, \cr
+#' \code{"manual"}: no clustering is necessary, researcher has specified cluster assignments.
 #'
 #' For all clustering methods, additional arguments to fine-tune clustering
 #' performance, such as the specific algorithm to be used, should be passed to
@@ -31,8 +31,8 @@
 #' \code{clusters} argument.
 #' @param type a character string giving the type of clustering method to be used.
 #' See Details.
-#' @param k the desired number of clusters. Required if \code{type = "hclust"},
-#' \code{type = "kmeans"}, or \code{type = "pam"}.
+#' @param k the desired number of clusters. Required if \code{type} is one of \code{"hclust"},
+#' \code{"diana"}, \code{"kmeans"}, or \code{"pam"}.
 #' @param epsilon radius of epsilon neighborhood. Required if \code{type = "dbscan"}.
 #' @param model a logical indicating whether clustering model output should be
 #' returned. Defaults to \code{FALSE}.
@@ -46,9 +46,9 @@
 #' @param ... arguments passed to other functions.
 #'
 #' @return If \code{model = TRUE}, \code{CPC()} returns a list with components
-#' containing output from the specified clustering function, all sums of squares,
-#' CPC, and adjusted CPC. If \code{model = FALSE}, \code{CPC()} returns a numeric
-#' vector of length 1 giving the CPC (if \code{adjust = FALSE}) or adjusted CPC (if
+#' containing output from the specified clustering function, all sums of squares, the
+#' CPC, the adjusted CPC, and associated standard errors. If \code{model = FALSE}, \code{CPC()} returns
+#' a numeric vector of length 1 giving the CPC (if \code{adjust = FALSE}) or adjusted CPC (if
 #' \code{adjust = TRUE}).
 #'
 #' @examples
@@ -60,19 +60,18 @@
 #' CPC(data, "manual", cols = 1:2, clusters = 3)
 #'
 #' @import stats
-#' @importFrom cluster pam
+#' @importFrom cluster pam diana
 #' @importFrom dbscan dbscan
-#'
 #' @export
 
 CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FALSE,
                 cols = NULL, clusters = NULL, ...) {
   data <- as.matrix(data)
   input <- data[colSums(!is.na(data)) > 0]
-  input <- as.matrix(na.omit(input))
+  input <- matrix(na.omit(input), ncol = ncol(data))
   cluster <- NULL
 
-  k <- ifelse(type %in% c("kmeans", "pam", "hclust"), k, 0)
+  k <- ifelse(!type %in% c("dbscan", "manual"), k, 0)
 
   if(length(unique(input)) < k){
     warning("More clusters than unique data points; NAs generated")
@@ -104,11 +103,13 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               TSS_dbscan <- SS(as.matrix(data_dbscan))
               TWSS_dbscan <- sum(WSS_dbscan)
               BSS_dbscan <- TSS_dbscan - TWSS_dbscan
+              n_i <- nrow(as.matrix(data_dbscan))
+              n_j <- ncol(as.matrix(data_dbscan))
+              n_k <- length(unique(new_dbscan$cluster))
               CPC <- BSS_dbscan/TSS_dbscan
-              CPC.adj <- 1 - (TWSS_dbscan/TSS_dbscan)*
-                ((nrow(as.matrix(data_dbscan)) - ncol(as.matrix(data_dbscan)))/
-                   (nrow(as.matrix(data_dbscan)) - ncol(as.matrix(data_dbscan))*
-                      length(unique(new_dbscan$cluster))))
+              CPC_sd <- sqrt((2*(n_j*n_k - n_j)*(n_i - n_j*n_k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_dbscan/TSS_dbscan)*((n_i - n_j)/(n_i - n_j*n_k))
+              CPC.adj_sd <- sqrt((2*(n_j*n_k - n_j))/((n_i - n_j*n_k)*(n_i - n_j + 1)))
 
               if(model){
                 list(cluster = output_dbscan$cluster,
@@ -119,7 +120,9 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
                      BSS = BSS_dbscan,
                      TSS = TSS_dbscan,
                      CPC = CPC,
-                     CPC.adj = CPC.adj)
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
               }
 
               else{
@@ -149,9 +152,12 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               TSS_hclust <- SS(input)
               TWSS_hclust <- sum(WSS_hclust)
               BSS_hclust <- TSS_hclust - TWSS_hclust
+              n_i <- nrow(input)
+              n_j <- ncol(input)
               CPC <- BSS_hclust/TSS_hclust
-              CPC.adj <- 1 - (TWSS_hclust/TSS_hclust)*
-                ((nrow(input) - ncol(input))/(nrow(input) - ncol(input)*k))
+              CPC_sd <- sqrt((2*(n_j*k - n_j)*(n_i - n_j*k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_hclust/TSS_hclust)*((n_i - n_j)/(n_i - n_j*k))
+              CPC.adj_sd <- sqrt((2*(n_j*k - n_j))/((n_i - n_j*k)*(n_i - n_j + 1)))
 
               if(model){
                 list(merge = output_hclust$merge,
@@ -167,7 +173,60 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
                      BSS = BSS_hclust,
                      TSS = TSS_hclust,
                      CPC = CPC,
-                     CPC.adj = CPC.adj)
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
+              }
+
+              else{
+                if(adjust){
+                  CPC.adj
+                }
+
+                else{
+                  CPC
+                }
+              }
+            },
+            diana = {
+              input <- apply(input, 2, as.numeric)
+              input_dist <- dist(input)
+              output_diana <- diana(input_dist, ...)
+              cut_diana <- as.data.frame(cutree(output_diana, k = k))
+              colnames(cut_diana) <- "cluster"
+              new_diana <- cbind(input, cut_diana)
+              WSS_diana <- c()
+
+              for (i in 1:k) {
+                WSS <- SS(new_diana[new_diana$cluster == i,])
+                WSS_diana <- c(WSS_diana, WSS)
+              }
+
+              TSS_diana <- SS(input)
+              TWSS_diana <- sum(WSS_diana)
+              BSS_diana <- TSS_diana - TWSS_diana
+              n_i <- nrow(input)
+              n_j <- ncol(input)
+              CPC <- BSS_diana/TSS_diana
+              CPC_sd <- sqrt((2*(n_j*k - n_j)*(n_i - n_j*k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_diana/TSS_diana)*((n_i - n_j)/(n_i - n_j*k))
+              CPC.adj_sd <- sqrt((2*(n_j*k - n_j))/((n_i - n_j*k)*(n_i - n_j + 1)))
+
+              if(model){
+                list(order = output_diana$order,
+                     height = output_diana$height,
+                     dc = output_diana$merge,
+                     diss = output_diana$diss,
+                     call = output_diana$call,
+                     data = new_diana,
+                     WSS = WSS_diana,
+                     TWSS = TWSS_diana,
+                     BSS = BSS_diana,
+                     TSS = TSS_diana,
+                     CPC = CPC,
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
               }
 
               else{
@@ -187,9 +246,13 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               colnames(cluster_kmeans) <- "cluster"
               new_kmeans <- cbind(input, cluster_kmeans)
 
+              n_i <- nrow(input)
+              n_j <- ncol(input)
               CPC <- output_kmeans$betweenss/output_kmeans$totss
-              CPC.adj <- 1 - (output_kmeans$tot.withinss/output_kmeans$totss)*
-                ((nrow(input) - ncol(input))/(nrow(input) - ncol(input)*k))
+              CPC_sd <- sqrt((2*(n_j*k - n_j)*(n_i - n_j*k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 -
+                (output_kmeans$tot.withinss/output_kmeans$totss)*((n_i - n_j)/(n_i - n_j*k))
+              CPC.adj_sd <- sqrt((2*(n_j*k - n_j))/((n_i - n_j*k)*(n_i - n_j + 1)))
 
               if(model){
                 list(centers = output_kmeans$centers,
@@ -202,7 +265,9 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
                      BSS = output_kmeans$betweenss,
                      TSS = output_kmeans$totss,
                      CPC = CPC,
-                     CPC.adj = CPC.adj)
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
               }
 
               else{
@@ -231,9 +296,12 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               TSS_pam <- SS(input)
               TWSS_pam <- sum(WSS_pam)
               BSS_pam <- TSS_pam - TWSS_pam
+              n_i <- nrow(input)
+              n_j <- ncol(input)
               CPC <- BSS_pam/TSS_pam
-              CPC.adj <- 1 - (TWSS_pam/TSS_pam)*
-                ((nrow(input) - ncol(input))/(nrow(input) - ncol(input)*k))
+              CPC_sd <- sqrt((2*(n_j*k - n_j)*(n_i - n_j*k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_pam/TSS_pam)*((n_i - n_j)/(n_i - n_j*k))
+              CPC.adj_sd <- sqrt((2*(n_j*k - n_j))/((n_i - n_j*k)*(n_i - n_j + 1)))
 
               if(model){
                 list(medoids = output_pam$medoids,
@@ -250,7 +318,62 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
                      BSS = BSS_hclust,
                      TSS = TSS_hclust,
                      CPC = CPC,
-                     CPC.adj = CPC.adj)
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
+              }
+
+              else{
+                if(adjust){
+                  CPC.adj
+                }
+
+                else{
+                  CPC
+                }
+              }
+            },
+            hclust = {
+              input <- apply(input, 2, as.numeric)
+              input_dist <- dist(input)
+              output_hclust <- hclust(input_dist, ...)
+              cut_hclust <- as.data.frame(cutree(output_hclust, k = k))
+              colnames(cut_hclust) <- "cluster"
+              new_hclust <- cbind(input, cut_hclust)
+              WSS_hclust <- c()
+
+              for (i in 1:k) {
+                WSS <- SS(new_hclust[new_hclust$cluster == i,])
+                WSS_hclust <- c(WSS_hclust, WSS)
+              }
+
+              TSS_hclust <- SS(input)
+              TWSS_hclust <- sum(WSS_hclust)
+              BSS_hclust <- TSS_hclust - TWSS_hclust
+              n_i <- nrow(input)
+              n_j <- ncol(input)
+              CPC <- BSS_hclust/TSS_hclust
+              CPC_sd <- sqrt((2*(n_j*k - n_j)*(n_i - n_j*k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_hclust/TSS_hclust)*((n_i - n_j)/(n_i - n_j*k))
+              CPC.adj_sd <- sqrt((2*(n_j*k - n_j))/((n_i - n_j*k)*(n_i - n_j + 1)))
+
+              if(model){
+                list(merge = output_hclust$merge,
+                     height = output_hclust$height,
+                     order = output_hclust$order,
+                     labels = output_hclust$labels,
+                     method = output_hclust$method,
+                     call = output_hclust$call,
+                     dist.method = output_hclust$dist.method,
+                     data = new_hclust,
+                     WSS = WSS_hclust,
+                     TWSS = TWSS_hclust,
+                     BSS = BSS_hclust,
+                     TSS = TSS_hclust,
+                     CPC = CPC,
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
               }
 
               else{
@@ -264,14 +387,14 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               }
             },
             manual = {
-              input <- na.omit(CPCdata.frame(data = data, cols = cols, clusters = clusters))
-              data_manual <- as.matrix(input[,-which(colnames(input) == "cluster")])
+              input <- CPCdata.frame(data = data, cols = cols, clusters = clusters)
+              data_manual <- as.matrix(input[, -ncol(input)])
               data_manual <- apply(data_manual, 2, as.numeric)
               WSS_manual <- c()
 
               for (i in unique(input$cluster)) {
                 data_temp <- input[input$cluster == i,]
-                data_temp <- as.matrix(data_temp[,-which(colnames(input) == "cluster")])
+                data_temp <- as.matrix(data_temp[, -ncol(input)])
                 data_temp <- apply(data_temp, 2, as.numeric)
                 WSS <- SS(as.matrix(data_temp))
                 WSS_manual <- c(WSS_manual, WSS)
@@ -280,11 +403,13 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
               TSS_manual <- SS(as.matrix(data_manual))
               TWSS_manual <- sum(WSS_manual)
               BSS_manual <- TSS_manual - TWSS_manual
+              n_i <- nrow(input)
+              n_j <- ncol(input)
+              n_k <- length(unique(input$cluster))
               CPC <- BSS_manual/TSS_manual
-              CPC.adj <- 1 - (TWSS_manual/TSS_manual)*
-                ((nrow(as.matrix(data_manual)) - ncol(as.matrix(data_manual)))/
-                   (nrow(as.matrix(data_manual)) - ncol(as.matrix(data_manual))*
-                      length(unique(input$cluster))))
+              CPC_sd <- sqrt((2*(n_j*n_k - n_j)*(n_i - n_j*n_k))/(((n_i - n_j)^2)*(n_i - n_j + 1)))
+              CPC.adj <- 1 - (TWSS_manual/TSS_manual)*((n_i - n_j)/(n_i - n_j*n_k))
+              CPC.adj_sd <- sqrt((2*(n_j*n_k - n_j))/((n_i - n_j*n_k)*(n_i - n_j + 1)))
 
               if(model){
                 list(data = input,
@@ -293,7 +418,9 @@ CPC <- function(data, type, k = NULL, epsilon = NULL, model = FALSE, adjust = FA
                      BSS = BSS_manual,
                      TSS = TSS_manual,
                      CPC = CPC,
-                     CPC.adj = CPC.adj)
+                     CPC_sd = CPC_sd,
+                     CPC.adj = CPC.adj,
+                     CPC.adj_sd = CPC.adj_sd)
               }
 
               else{
